﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Console : MonoBehaviour
{
    string log;
    bool toggle = false;

    Vector2 guiScroll = Vector2.zero;

    // Coroutine and coroutine visualizations.
    Coroutine coroutTest = null;
    const string coroutineTestString = "----------------------------------------";
    int coroutineTestIdx = 0;

    // Variables for counting the number of FixedUpdate() calls per frame.
    int fixedCalls = 0;
    int fixedCallIncr = 0;

    // Signals for calling exceptions
    bool exceptionOnUpdate = false;
    bool exceptionOnCoroutine = false;

    private void Awake()
    {
        // Capture log messages.
        Application.logMessageReceived += this.LogCallback;
    }

    // The callback to capture log messages (See Awake()).
    void LogCallback(string condition, string stackTrace, LogType type)
    { 
        log += type.ToString() + " : " + condition + "\n";
    }

    void Start()
    {
        this.ListHREFParameters();
    }

    
    void Update()
    {
        // Update the frame flipper.
        this.toggle = !toggle;

        // Store the last fixedCall counter for last frame, 
        // and reset the counter
        this.fixedCalls = this.fixedCallIncr;
        this.fixedCallIncr = 0;

        if (this.exceptionOnUpdate == true)
        { 
            this.exceptionOnUpdate = false;
            throw new System.Exception("Threw an exception on Update()");
        }
    }

    private void FixedUpdate()
    {
        ++this.fixedCallIncr;
    }

    string ConvertString(string start)
    {  
        return start.Replace("A", "ZZ");
    }

    private void OnGUI()
    {
        this.guiScroll = GUILayout.BeginScrollView(this.guiScroll, GUILayout.Width(400.0f));

        GUILayout.BeginVertical(GUI.skin.box);

            // Timing
            //////////////////////////////////////////////////
            GUILayout.Label("Timing", GUI.skin.box);
                GUILayout.Label($"Time.timeScale : {Time.timeScale}");
                GUILayout.BeginHorizontal();
                Time.timeScale = GUILayout.HorizontalSlider(Time.timeScale, 0.0f, 2.0f);
                if(GUILayout.Button("1.0", GUILayout.ExpandWidth(false)) == true)
                    Time.timeScale = 1.0f;
                GUILayout.EndHorizontal();
                GUILayout.Label($"Time.deltaTime : {Time.deltaTime}");
                GUILayout.Label($"Time.fixedDeltaTime : {Time.fixedDeltaTime}");
                GUILayout.Label($"Time.time : {Time.time}");
                GUILayout.Label($"Fixed per frame : {this.fixedCalls}");
            GUILayout.EndVertical();

            // Frame change test
            //////////////////////////////////////////////////
            GUILayout.BeginVertical(GUI.skin.box);
                GUILayout.Label("Frame Flipping", GUI.skin.box);
                GUILayout.Label(this.toggle ? "Boowap!" : this.ConvertString("ABCABC"));
            GUILayout.EndVertical();

            // Debug.Log() Stuff
            //////////////////////////////////////////////////
            GUILayout.BeginVertical(GUI.skin.box);
                GUILayout.Label("Test Debug.Log*() functions", GUI.skin.box);
                GUILayout.BeginHorizontal();
                    if(GUILayout.Button("Log") == true)
                        Debug.Log("A Debug.Log() call");
                    if(GUILayout.Button("Warning") == true)
                        Debug.LogWarning("A Debug.Warning() call");
                    if (GUILayout.Button("Error") == true)
                        Debug.LogError("A Debug.Error() call");
                    if(GUILayout.Button("Clear") == true)
                        this.log = string.Empty;
                GUILayout.EndHorizontal();
                GUILayout.TextField( this.log);
            GUILayout.EndVertical();

            GUILayout.BeginVertical(GUI.skin.box);
                GUILayout.Label("Exception Testing", GUI.skin.box);
                if(GUILayout.Button("Exception on Update()") == true)
                    this.exceptionOnUpdate = true;
                if(GUILayout.Button("Exception on coroutine") == true)
                    this.exceptionOnCoroutine = true;
            GUILayout.EndVertical();

            // Coroutine testing
            //////////////////////////////////////////////////
            GUILayout.BeginVertical(GUI.skin.box);
                GUILayout.Label("Coroutine Testing", GUI.skin.box);
                System.Text.StringBuilder strCoroutMoment = new System.Text.StringBuilder(coroutineTestString);
                strCoroutMoment[this.coroutineTestIdx] = '#';
                GUILayout.Label(strCoroutMoment.ToString());

                if(GUILayout.Button("Start Coroutine") == true)
                    this.StartTestCoroutine();
                if(GUILayout.Button("Stop Coroutine") == true)
                    this.StopTestCoroutine();
            GUILayout.EndVertical();

        // Misc testing
        //////////////////////////////////////////////////
        //GUILayout.BeginVertical(GUI.skin.box);
        //GUILayout.Label("Misc", GUI.skin.box);
        //    if(GUILayout.Button("") == true)
        //        this.ListHREFParameters();
        //GUILayout.EndVertical();

        GUILayout.EndScrollView();
    }

    void StartTestCoroutine()
    { 
        if(this.coroutTest != null)
        {
            this.StopCoroutine(this.coroutTest);
            this.coroutTest = null;
        }
        this.coroutTest = this.StartCoroutine(this.TestCoroutine());
    }

    void ListHREFParameters()
    { 
        Dictionary<string, string> prms = ParamParse.GetBrowserParameters();
        if(prms.Count == 0)
            return;

        string output = "Listing Link Parameters:\n";
        foreach(KeyValuePair<string, string> kvp in prms)
        { 
            if(string.IsNullOrEmpty(kvp.Value) == true)
                output += "\tKeyword " + kvp.Key + "\n";
            else
                output += "\tMapping " + kvp.Key + " with value " + kvp.Value + "\n";
        }
        this.log += output;
    }

    void StopTestCoroutine()
    { 
        if(this.coroutTest != null)
        { 
            Coroutine cpy = this.coroutTest;
            this.coroutTest = null;
            this.StopCoroutine(cpy);
        }
    }

    IEnumerator TestCoroutine()
    { 
        while(true)
        {
            if(this.exceptionOnCoroutine == true)
            { 
                this.exceptionOnCoroutine = false;
                throw new System.Exception("Threw an exception in a coroutine.");
            }

            this.coroutineTestIdx = (this.coroutineTestIdx + 1) % coroutineTestString.Length;
            yield return null;
        }
    }
}
